@@ -2,14 +2,11 @@
 ### Endemic-epidemic modelling for univariate or multivariate
 ### time series of infectious disease counts (data class "sts")
 ###
-### Copyright (C) 2010-2012 Michaela Paul, 2012-2016,2019-2020 Sebastian Meyer
+### Copyright (C) 2010-2012 Michaela Paul, 2012-2016,2019-2023 Sebastian Meyer
 ###
 ### This file is part of the R package "surveillance",
 ### free software under the terms of the GNU General Public License, version 2,
 ### a copy of which is available at https://www.R-project.org/Licenses/.
-###
-### $Revision: 2564 $
-### $Date: 2020-10-11 23:31:03 +0200 (Sun, 11. Oct 2020) $
 ################################################################################
 
 ## Error message issued in loglik, score and fisher functions upon NA parameters
@@ -173,7 +170,7 @@ setControl <- function (control, stsObj)
   defaultControl <- eval(formals(hhh4)$control)
   environment(defaultControl$ar$f) <- environment(defaultControl$ne$f) <-
       environment(defaultControl$end$f) <- .GlobalEnv
-  control <- modifyList(defaultControl, control)
+  control <- modifyList(defaultControl, control, keep.null = TRUE)
 
   ## check that component specifications are list objects
   for (comp in c("ar", "ne", "end")) {
@@ -222,7 +219,7 @@ setControl <- function (control, stsObj)
 
   if (control$ne$inModel) {
       if (nUnit == 1)
-          stop("\"ne\" component requires a multivariate 'stsObj'")
+          warning("\"ne\" component requires a multivariate 'stsObj'")
       ## if ar$f is a matrix it includes neighbouring units => no "ne" component
       if (control$ar$isMatrix)
           stop("there must not be an extra \"ne\" component ",
@@ -261,7 +258,7 @@ setControl <- function (control, stsObj)
           if (!identical(dim(control[[comp]]$offset), dim(stsObj)))
               stop("'control$",comp,"$offset' must be a numeric matrix of size ",
                    nTime, "x", nUnit)
-          if (any(is.na(control[[comp]]$offset)))
+          if (anyNA(control[[comp]]$offset))
               stop("'control$",comp,"$offset' must not contain NA values")
       } else if (!identical(as.numeric(control[[comp]]$offset), 1)) {
           stop("'control$",comp,"$offset' must either be 1 or a numeric ",
@@ -506,6 +503,9 @@ checkFormula <- function(f, component, data, stsObj)
   vars <- as.list(attr(term,"variables"))[-1] # first element is "list"
   nVars <- length(vars)
 
+  if (nVars > 0 && any(attr(term, "order") > 1))
+      warning("interaction terms are not implemented")
+
   # begin with intercept
   res <- if (intercept.all) {
       c(fe(1), list(offsetComp=component))
@@ -921,7 +921,7 @@ sizeHHH <- function (theta, model, subset = model$subset)
 
 penLogLik <- function(theta, sd.corr, model, attributes=FALSE)
 {
-  if(any(is.na(theta))) stop("NAs in regression parameters.", ADVICEONERROR)
+  if(anyNA(theta)) stop("NAs in regression parameters.", ADVICEONERROR)
 
   ## unpack model
   subset <- model$subset
@@ -986,7 +986,7 @@ penLogLik <- function(theta, sd.corr, model, attributes=FALSE)
 
 penScore <- function(theta, sd.corr, model)
 {
-  if(any(is.na(theta))) stop("NAs in regression parameters.", ADVICEONERROR)
+  if(anyNA(theta)) stop("NAs in regression parameters.", ADVICEONERROR)
 
   ## unpack model
   subset <- model$subset
@@ -1100,7 +1100,7 @@ penScore <- function(theta, sd.corr, model)
 
 penFisher <- function(theta, sd.corr, model, attributes=FALSE)
 {
-  if(any(is.na(theta))) stop("NAs in regression parameters.", ADVICEONERROR)
+  if(anyNA(theta)) stop("NAs in regression parameters.", ADVICEONERROR)
 
   ## unpack model
   subset <- model$subset
@@ -1491,22 +1491,14 @@ getSigma <- function(sd, correlation, dimSigma, dimBlocks, Sigmai=NULL){
 ## identical in marScore() and marFisher()
 marLogLik <- function(sd.corr, theta, model, fisher.unpen=NULL, verbose=FALSE){
 
-  dimVar <- model$nVar
-  dimCorr <- model$nCorr
   dimSigma <- model$nSigma
-
   if(dimSigma == 0){
     return(-Inf)
   }
+  if(anyNA(sd.corr)) stop("NAs in variance parameters.", ADVICEONERROR)
 
-  if(any(is.na(sd.corr))){
-    # in order to avoid nlminb from running into an infinite loop (cf. bug
-    # report #15052), we have to emergency stop() in this case.
-    # As of R 2.15.2, nlminb() throws an error if it receives NA from
-    # any of the supplied functions.
-    stop("NAs in variance parameters.", ADVICEONERROR)
-  }
-
+  dimVar <- model$nVar
+  dimCorr <- model$nCorr
   sd   <- head(sd.corr,dimVar)
   corr <- tail(sd.corr,dimCorr)
 
@@ -1550,16 +1542,14 @@ marLogLik <- function(sd.corr, theta, model, fisher.unpen=NULL, verbose=FALSE){
 
 marScore <- function(sd.corr, theta, model, fisher.unpen=NULL, verbose=FALSE){
 
-  dimVar <- model$nVar
-  dimCorr <- model$nCorr
   dimSigma <- model$nSigma
-
   if(dimSigma == 0){
     return(numeric(0L))
   }
+  if(anyNA(sd.corr)) stop("NAs in variance parameters.", ADVICEONERROR)
 
-  if(any(is.na(sd.corr))) stop("NAs in variance parameters.", ADVICEONERROR)
-
+  dimVar <- model$nVar
+  dimCorr <- model$nCorr
   sd   <- head(sd.corr,dimVar)
   corr <- tail(sd.corr,dimCorr)
 
@@ -1628,16 +1618,14 @@ marScore <- function(sd.corr, theta, model, fisher.unpen=NULL, verbose=FALSE){
 
 marFisher <- function(sd.corr, theta, model, fisher.unpen=NULL, verbose=FALSE){
 
-  dimVar <- model$nVar
-  dimCorr <- model$nCorr
   dimSigma <- model$nSigma
-
   if(dimSigma == 0){
     return(matrix(numeric(0L),0L,0L))
   }
+  if(anyNA(sd.corr)) stop("NAs in variance parameters.", ADVICEONERROR)
 
-  if(any(is.na(sd.corr))) stop("NAs in variance parameters.", ADVICEONERROR)
-
+  dimVar <- model$nVar
+  dimCorr <- model$nCorr
   sd   <- head(sd.corr,dimVar)
   corr <- tail(sd.corr,dimCorr)
 
@@ -1936,7 +1924,7 @@ updateParams_nlm <- function (start, ll, sc, fi, ..., control)
     ## run the optimization
     res <- do.call("nlm", args=c(alist(p=start, f=negllscfi, ...), control))
     ## Done
-    list(par=res$estimate, ll=-res$minimum,
+    list(par=setNames(res$estimate, names(start)), ll=-res$minimum,
          rel.tol=getRelDiff(res$estimate, start),
          convergence=as.numeric(res$code>2), message=res$message)
     ## nlm returns convergence status in $code, 1-2 indicate convergence,
@@ -1981,7 +1969,7 @@ checkParBounds <- function (par, lower, upper)
 defaultOptimControl <- function (method = "nlminb", lower = -Inf, upper = Inf,
                                  iter.max = NULL, verbose = 0)
 {
-    if (is.null(iter.max)) iter.max <- 20 + 280*(method=="Nelder-Mead")
+    if (is.null(iter.max)) iter.max <- 20 + 480*(method=="Nelder-Mead")
     lowVerbose <- verbose %in% 0:2
     luOptimMethod <- method %in% c("Brent", "L-BFGS-B")
     defaults.nr <- list(scoreTol=1e-5, paramTol=1e-7, F.inc=0.01, stepFrac=0.5,
@@ -2110,20 +2098,12 @@ fitHHH <- function(theta, sd.corr, model,
                   parVar$rel.tol, "\n")
 
           if(parVar$convergence!=0) {
-              if (!is.null(parVar$message)) print(parVar$message)
+              if (!is.null(parVar$message))
+                  cat("! Non-convergence message from optimizer:",
+                      parVar$message, "\n")
               cat("Update of variance parameters in iteration ", i, " unreliable\n")
           }
 
-          ## NA values in sd.corr cause a stop() already in marLogLik()
-          ## if(any(is.na(parVar$par))){
-          ##     updateVarianceControl[[1]] <- "updateParams_optim"
-          ##     updateVarianceControl[[2]]$method <-
-          ##         if (length(sd.corr) == 1L) "Brent" else "Nelder-Mead"
-          ##     cat("  WARNING: at least one updated variance parameter is not a number\n",
-          ##         "\t-> NO UPDATE of variance\n",
-          ##         "\t-> SWITCHING to robust", dQuote(updateVarianceControl[[2]]$method),
-          ##         "for variance updates\n")
-          ## } else
           sd.corr <- parVar$par
 
           ## overall convergence ?
@@ -2143,12 +2123,11 @@ fitHHH <- function(theta, sd.corr, model,
         "Optimization converged" else "Optimization DID NOT CONVERGE", "\n\n")
   }
 
-  ll <- penLogLik(theta=theta,sd.corr=sd.corr,model=model)
-  fisher <- penFisher(theta=theta,sd.corr=sd.corr,model=model)
+  ll     <- penLogLik(theta, sd.corr, model)
+  fisher <- penFisher(theta, sd.corr, model, attributes = TRUE)
   dimnames(fisher) <- list(names(theta), names(theta))
-  margll <- marLogLik(sd.corr=sd.corr, theta=theta, model=model)
-  fisher.var <- marFisher(sd.corr=sd.corr, theta=theta, model=model,
-                          fisher.unpen=fisher.unpen)
+  margll     <- marLogLik(sd.corr, theta, model, attr(fisher, "fisher"))
+  fisher.var <- marFisher(sd.corr, theta, model, attr(fisher, "fisher"))
   dimnames(fisher.var) <- list(names(sd.corr), names(sd.corr))
 
   list(theta=theta, sd.corr=sd.corr,
