@@ -6,9 +6,6 @@
 ### This file is part of the R package "surveillance",
 ### free software under the terms of the GNU General Public License, version 2,
 ### a copy of which is available at https://www.R-project.org/Licenses/.
-###
-### $Revision: 2963 $
-### $Date: 2023-03-13 23:24:10 +0100 (Mon, 13. Mar 2023) $
 ################################################################################
 
 
@@ -41,7 +38,7 @@ discpoly <- function (center, radius, npoly = 64,
             ##gpcWarning()
             pts <- list(list(x=x, y=y, hole=hole))
             if (isClass("gpc.poly") ||
-                (gpclibCheck() && requireNamespace("gpclib"))) {
+                (gpclibCheck(FALSE) && requireNamespace("gpclib"))) {
                 new("gpc.poly", pts = pts)
             } else {
                 warning("formal class \"gpc.poly\" not available")
